/** Piotr Rolewski G1ISI
Program pokazujacy zastosowanie dziedziczenia prywatnego i chrononego */

/** W C++ mozna dziedziczyc w sposob prywatny, jesli pominiemy specyfikator public badz tez jawnie 
wpisujac slowo private. W dziedziczenie chronione polega na uzyciu slowa kluczowego protected.
Podczas dziedziczenia prywatnego wszystkie skladowe publiczne i chronione klasy podstawowej staja sie skladowymi
PRYWATNYMI klasy pochodnej. Jest ono przydatne, gdy chcemy ukryc czesc funkcji zawartych w klasie podstawowej.
Dziedziczenie chronione jest odmiana dziedziczenia prywatnego. Podczas dziedziczenia chronionego skladowe publiczne
i chronione klasy podstawowej staja sie skladowymi CHRONIONYMI klasy pochodnej. Oznacza to, ze skladowe te beda dostepne
rowniez na kolejnym poziomie hierarchii dziedziczenia (regula ta nie sprawdza sie dla dziedziczenia prywatnego).
Zarowno dziedziczenie prywatne jak i chronione odpowiada relacji "ma cos" dlatego czasami latwiej jest zastosowac relacje
zawierania. Jesli jednak w klasie pochodnej chcemy miec dostep do danych chronionych lub chcemy przedefiniowac funkcje 
wirtualna latwiej korzystac z dziedziczenia.*/

#include <iostream>
#include <string>

using namespace std;

class Engine{
public:
	//konstruktor klasy Engine
	Engine(double hourses, double cap, int num):hoursePower(hourses), capacity(cap), serialNumber(num){
		oilLevel = 0;
		state = false;
	}
	//metoda wirtualna, ktora nie moglaby byc przedefioniowana za pomoca relacji zawierania
	virtual bool chceckLiquids()const {return oilLevel>50;}
protected:						//specyfikator protected umozliwia dostep klasom pochodnym
	int oilLevel;				//poziom oleju w silniku
	bool state;					//stan: wlaczony/wylaczony
private:						//dostepne jedynie w klasie bazowej Engine
	double hoursePower;			//moc silnika	
	double capacity;			//pojemno silnika
	int serialNumber;			//numer seryjny silnika
};

//dziedziczenie chronione
class Vehicle1: protected Engine{
public:
	//konstruktor klasy Vehicle1
	Vehicle1(double hourses, double cap, int num, double w, double com, string own):Engine(hourses, cap, num){
		weight = w;					//dostep do wlasnych danych chronionych
		combustion = com;	
		fuelLevel = 0;				//dostep do danych prywatnych
		owner = own;
		//hoursePower = hourses - 1;	//BLAD brak dostepu do danych prywatnych klasy bazowej
	}
	//przedefiniowanie funkcji klasy bazowej
	virtual bool chceckLiquids()const {return (oilLevel>80 && fuelLevel>50);}
	virtual void  fillUp(){
		oilLevel = 100;				//dostep do danej chronionej klasy Engine
		fuelLevel = 100;
	}
	void turn(bool on) {state=on;}	//dostep do danej chronionej klasy Engine
protected:
	double weight;						//masa pojazdu
	double combustion;					//spalanie pojazdu
private:
	int fuelLevel;						//poziom paliwa		
	string owner;						//nazwa wlasciciela
};

//to samo zrealizowane za pomoca dziedziczenia prywatnego
class Vehicle2: private Engine{
public:
	//konstruktor klasy Vehicle2
	Vehicle2(double hourses, double cap, int num, double w, double com, string own):Engine(hourses, cap, num){
		weight = w;					//dostep do wlasnych danych chronionych
		combustion = com;	
		fuelLevel = 0;				//dostep do danych prywatnych
		owner = own;
	}
	//przedefiniowanie funkcji klasy bazowej
	virtual bool chceckLiquids()const {return (oilLevel>80 && fuelLevel>50);}
	virtual void  fillUp(){
		oilLevel = 100;				//dostep do danej chronionej klasy Engine
		fuelLevel = 100;
	}
	void turn(bool on){state=on;}	//dostep do danej chronionej klasy Engine
protected:
	double weight;						//masa pojazdu
	double combustion;					//spalanie pojazdu
private:
	int fuelLevel;						//poziom paliwa		
	string owner;						//nazwa wlasciciela
};

//roznice w dziedziczeniu prywatnym wystepuja na dalszych poziomach dziedziczenia 
//klasa Vehicle1 dziedziczyla w sposob chroniony po klasie Engine
//W klasie Car nadal mozliwy jest dostep do danych chronionych klasy Engine
class Car: public Vehicle1{
public:
	Car(double hourses, double cap, int num, double w, double com, string own):Vehicle1(hourses, cap, num, w, com, own){
	windSreenLiquid = 0;}
	void fillUp(){
	windSreenLiquid = 100;
	oilLevel = 100;				//dostep do danej chronionej klasy Engine
	//fluelLevel = 150;			//BLAD brak dostepu do danej prywatnej klasy Vehicle1
	}	
private:
	int windSreenLiquid;

};
//klasa Vehicle2 dziedziczyla w sposob prywatny po klasie Engine
//wynikiem jest brak dostepu do danej chronionej klasy Engine
class Boat: public Vehicle2{
public:
	Boat(double hourses, double cap, int num, double w, double com, string own):Vehicle2(hourses, cap, num, w, com, own){
		water = 0;}
		void fillUp(){
		water = 100;
		//oilLevel = 200;			//BLAD brak dostepu do danej chronionej klasy Engine
		//fluelLevel = 200;			//BLAD brak dostepu do danej prywatnej klasy Vehicle1
		}
private:
	int water;
};
int main()
{
	Car car(200.0, 2.5, 1234, 1050.5, 7.7, "Jan Kowalski");
	car.fillUp();
	car.turn(true);
	Boat boat(300.0, 4.5, 12534, 2050.5, 17.7, "Jan Nowak");
	boat.fillUp();
	boat.turn(true);
}